/*  hivemind -- a client for cadaver
    Copyright (C) 2000 Kari Pahula

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#ifndef CONTROL_H
#define CONTROL_H

#include <pthread.h>
#include <unistd.h>
#include <stdio.h>

#include <vector>
#include <set>
#include <string>

class Communicate;
class Cyborg;

#include "control.H"
#include "turf.H"

class Cyborg {
public:
  Cyborg (const char *&);

  enum state {IDLE, GOTO, FIGHT, TAKE, DROP, EAT, CONVERT, BURN};
  enum item {WATER, GRAIN, GRASS, CARCASS, HAY, MEAT, BREAD, FOREST};

  // Some fields are omitted, since they are cosmetic
  bool gender;		// Males are true
  /* It's necessary to be able to change the state manually, since the server
     may not always be updated on the next tick.  */
  mutable enum state state;
  enum item item;
  int targetid, id, ownerid;
  float x, y, targetx, targety, age;
  unsigned char fertility, health, fightstrenght;
  unsigned short grain, grass, carcass, meat, bread, forest;

  bool operator== (const Cyborg &cyborg) const {return cyborg.id == id;}
  bool operator== (const Cyborg * const cyborgptr) const
    {return cyborgptr->id == id;}

  inline float dist (const Cyborg &cyborg) const
    {float xdiff = x-cyborg.x, ydiff = y-cyborg.y;
    return xdiff*xdiff+ydiff*ydiff;}

private:
  int pseudofloat (const char **);
};

class Communicate {
public:
  Communicate (int &, char **&);

  void handler ();
  void add_component (Component *comp) {componentlist.push_back (comp);}
  bool same_owner (const int &otherid) const {return otherid == id;}

  // Functions, which communicate with the server.
  void take (const int &, const enum Cyborg::item &) const;
  void drop (const int &, const enum Cyborg::item &) const;
  void eat (const int &, const enum Cyborg::item &) const;
  void convert (const int &, const enum Cyborg::item &) const;
  void burn (const int &, const enum Cyborg::item &) const;
  void idle (const int &) const;
  void move (const int &, const float &, const float &) const;
  void fight (const int &, const int &) const;
  void sex (const int &, const int &) const;

  Turf turf;
  Control control;

private:
  Communicate (const Communicate &);		// no, you can't copy me
  void operator= (const Communicate &);		// get your own connection!

  int fd;		// to the server
  int id;		// client id
  static const int pulse_interval = 100000;

  void wait_input_and_query (bool &);
  void readline (string &);
  inline void write_serv (const char * const, const size_t &) const;
  inline void query () {write_serv ("QN\n", 3);}

  vector<Component *> componentlist;
  // Set of existing cyborgs, used to check for dead cyborgs
  void cyborg_update (const char *);
  set<int> exists;
};
#endif
