/*  hivemind -- a client for cadaver
    Copyright (C) 2000 Kari Pahula

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#ifndef COMPONENT_H
#define COMPONENT_H

#include <stdio.h>
#ifdef HAVE_LIBPTHREAD
#include <pthread.h>
#endif

#include <list>
#include <vector>

class RWlock;
class Control;
class Component;
// from communicate.H
class Communicate;
class Cyborg;

#ifdef HAVE_LIBPTHREAD
class RWlock {
public:
  RWlock ();
  void get_read ();
  void get_write ();
  void release_read ();
  void release_write ();
private:
  bool write_lock;
  int n_lock;
  pthread_mutex_t nlock_mutex;
  pthread_cond_t w_wait, r_wait;
};
#endif

class Control {
#ifdef HAVE_LIBPTHREAD
private:
  pthread_mutex_t def_mutex;
  pthread_cond_t def_cond;
#endif
public:
  Control (const Communicate *c) {comm = c;
#ifdef HAVE_LIBPTHREAD
  pthread_mutex_init (&def_mutex, NULL);
  pthread_cond_init (&def_cond, NULL);
#endif
  }
  ~Control () {printf ("hivemind: final number of cyborgs: %d\n",
		       masterlist.size ());}
#ifdef HAVE_LIBPTHREAD
  void waitfor_default ();
  void iam_default (const Component * const);
  void giveme_enemy (const Component * const);
  RWlock list_lock;
  bool cc () {return concurrent;}
  void is_concurrent () {concurrent = 1;}
#else
  void waitfor_default () {}
  void iam_default (const Component *comp) {default_component = comp;}
  void giveme_enemy (const Component *comp) {default_enemycomp = comp;}
#endif
  void refresh_list (Component *);

  void update_cyborg (const Cyborg * const);	// Both new and updates
  void cyborg_lost (const int &);
private:
#ifdef HAVE_LIBPTHREAD
  bool concurrent;
#endif
  const Communicate *comm;
  const Component *default_component, *default_enemycomp;
  struct master_t {
    const Cyborg *cyborg;
    const Component *comp;
  };

  list<struct master_t> masterlist, graveyard;
  inline bool excavate_cyborg ()
    {
      for (list<struct master_t>::iterator iter = graveyard.begin ();
	   iter != graveyard.end (); iter++)
	if (iter->cyborg == NULL)
	  {
	    graveyard.erase (iter);
	    return true;
	  }
      return false;
    }
};

#include "communicate.H"

class Component {
  friend void Control::refresh_list (Component *);
public:
  virtual void handler () = 0;

  //static void enemy_sighted (const Cyborg &);
protected:
  const Communicate * comm;
  const Turf * turf;
  Control * control;

  vector<const Cyborg *> clist;		// Own cyborgs
  vector<const Cyborg *> graveyard;	// Dead cyborgs (only own!)
  vector<const Cyborg *> elist;		// Enemy cyborgs

  const Cyborg * nearest_cyborg (const int &, const int &, float &, void *,
				 bool (* const) (void *, const Turf *const,
						 const Cyborg *const, bool &));
#ifdef HAVE_LIBPTHREAD
public:
  template <class comp_type> void start_handler (comp_type);
private:
  template <class comp_type> static void * startup (void *);
#endif
};

#ifdef HAVE_LIBPTHREAD
template <class comp_type> void Component::start_handler (comp_type data)
{
  pthread_attr_t attr;
  pthread_t thread;
  pthread_attr_init (&attr);
  pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_DETACHED);
  pthread_create (&thread, &attr, &startup<comp_type>, data);
}
#endif

#endif
