/*  hivemind -- a client for cadaver
    Copyright (C) 2000 Kari Pahula

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#ifndef NOMAD_H
#define NOMAD_H

#include <vector>
#include <map>

#include "control.H"
#include "turf.H"
#include "communicate.H"

class Nomad : public Component {
public:
  Nomad (const Communicate *const, Control *const, const Turf *const);
  ~Nomad () {delete [] (map);}

  virtual void handler ();
private:
  map<unsigned int, char> cyborg_act;
  set<unsigned int> cyborg_eat;

  char *map;	// Assingments

  static bool ntest_idle (void *, const Turf *const, const Cyborg *const,
			  bool &);
  static bool ntest_idle_has_grain (void *, const Turf *const,
				    const Cyborg *const, bool &);
  void gatherer (const unsigned int &, const enum Cyborg::item &, const bool &,
		 const unsigned char &, bool &,
		 bool (* const) (void *, const Turf *const,
				 const Cyborg *const, bool &));
  vector<unsigned int> civilization;
  void planner ();

  short bread_stockpile_amount;

  inline void abort_action (const Cyborg &);

  void make_farm (const unsigned int &, const Cyborg &);

  void nourisher (const Cyborg &);
  void taker (const Cyborg &);
  void marshal (const Cyborg &);
  bool action_stopper (const Cyborg &);
  bool matchmaker (const Cyborg &, bool &);
  bool farmjob (const Cyborg &);
  void find_resource (const Cyborg &);

  static bool test_any_food (const Turf &, void *, unsigned const int &,
			     unsigned const int &);

  static bool test_weeds (const Turf &, void *, unsigned const int &,
			  unsigned const int &);
  static bool test_perimeter (const Turf &, void *, unsigned const int &,
			     unsigned const int &);
  static bool test_can_make_field (const Turf &, void *, unsigned const int &,
				   unsigned const int &);
  static bool test_harvest (const Turf &, void *, unsigned const int &,
			    unsigned const int &);
  static bool test_not_farm (const Turf &, void *, unsigned const int &,
			     unsigned const int &);
  static bool test_farm_more_bread (const Turf &, void *, unsigned const int &,
				    unsigned const int &);
  static bool test_farm_more_meat (const Turf &, void *, unsigned const int &,
				   unsigned const int &);

  static bool test_farm (const Turf &, void *, unsigned const int &,
			    unsigned const int &);
  static bool test_carcass (const Turf &, void *, unsigned const int &,
			    unsigned const int &);
  static bool test_grain (const Turf &, void *, unsigned const int &,
			  unsigned const int &);
  static bool test_food (const Turf &, void *, unsigned const int &,
			 unsigned const int &);
};
#endif
