/*  hivemind -- a client for cadaver
    Copyright (C) 2000 Kari Pahula

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "turf.H"
#include "communicate.H"
#include "nomad.H"

void Turf::init (int &fd, int &id)
{
  char buf[16];
  struct map_cell *ptr;

  if (read (fd, buf, 16) == -1)
    {
      perror ("hivemind");
      exit (1);
    }
  sscanf (buf, "%d %d %d", &_width, &_height, &id);
  _size = _width*_height;
  ptr = data = new struct map_cell [_size];
  for (int i = 0; i < _size; i++, ptr++)
    {
      ptr->forest = -1;
      ptr->grass = -1;
      ptr->grain = -1;
      ptr->carcass = -1;
      ptr->meat = -1;
      ptr->bread = -1;
    }
}

void Turf::update (const char *msg)
{
  while (*msg != '\n')
    {
      unsigned int x, y;
      x = strtol (msg, const_cast <char **> (&msg), 10);
      y = strtol (msg, const_cast <char **> (&msg), 10);
      int ref = coord (x, y);
      if (msg[1] == 'x')
	{
	  data[ref].visible = false;
	  msg += 2;
	}
      else
	{
	  data[ref].visible = true;
	  msg++;	// skip blank
	  msg++;	// skip water
	  data[ref].grain = *(msg++) - '0';
	  data[ref].grass = *(msg++) - '0';
	  data[ref].carcass = *(msg++) - '0';
	  msg++;	// skip hay
	  data[ref].meat = *(msg++) - '0';
	  data[ref].bread = *(msg++) - '0';
	  data[ref].forest = *(msg++) - '0';
	}
    }
}

unsigned int
Turf::find_nearest (unsigned int &x, unsigned int &y, unsigned const int
		    &radius, void *const comp, bool (* const test)
		    (const Turf &, void *, unsigned const int &,
		     unsigned const int &)) const
{
  int dir = rand () % 4;
  switch (dir)
    {
    case 0:
      for (unsigned int i = 1; i <= radius; i++)
	{
	  unsigned int j;
	  if (test (*this, comp, --x, y)) return i;
	  for (j = 1; j < i; j++)  if (test (*this, comp, --x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, ++y)) return i;
	}
      return 0;
    case 1:
      for (unsigned int i = 1; i <= radius; i++)
	{
	  unsigned int j;
	  if (test (*this, comp, x, --y)) return i;
	  for (j = 1; j < i; j++)  if (test (*this, comp, ++x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, --y)) return i;
	}
      return 0;
    case 2:
      for (unsigned int i = 1; i <= radius; i++)
	{
	  unsigned int j;
	  if (test (*this, comp, ++x, y)) return i;
	  for (j = 1; j < i; j++)  if (test (*this, comp, ++x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, --y)) return i;
	}
      return 0;
    case 3:
      for (unsigned int i = 1; i <= radius; i++)
	{
	  unsigned int j;
	  if (test (*this, comp, x, ++y)) return i;
	  for (j = 1; j < i; j++)  if (test (*this, comp, --x, ++y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, --x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, --y)) return i;
	  for (j = 1; j <= i; j++) if (test (*this, comp, ++x, ++y)) return i;
	}
      return 0;
    }
}
