/*  hivemind -- a client for cadaver
    Copyright (C) 2000 Kari Pahula

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
*/

#ifndef TURF_H
#define TURF_H

class Communicate;	// from communicate.H
class Cyborg;
class Component;	// from control.H

/* This class keeps a map of the terrain.  */
class Turf {
public:
  Turf (const Communicate *c) {comm = c;}
  ~Turf () {delete [] (data);}
  void init (int &, int &);

  void update (const char *);

  //bool check_stuff (const Cyborg &, const int &, const int &, const int &);
  inline int coord (const unsigned int &x, const unsigned int &y) const
    {if (x>=_width || y>=_height) return -1; return x+y*_width;}
  inline int reftox (const int &ref) const {return ref % _width;}
  inline int reftoy (const int &ref) const {return ref / _width;}
  inline unsigned int width () const {return _width;}
  inline unsigned int height () const {return _height;}
  inline unsigned int size () const {return _size;}

  struct map_cell {
    bool visible;
    signed char forest;		// Other fields have no effect on anything
    signed char grass;
    signed char grain;
    signed char carcass;
    signed char meat;
    signed char bread;
  };

  /* Returns distance.  */
  unsigned int find_nearest (unsigned int &, unsigned int &,
			     unsigned const int &, void *const, bool (* const)
			     (const Turf &, void *, const unsigned int &,
			      const unsigned int &)) const;

  inline const struct map_cell * read_map (unsigned const int &x, unsigned
					   const int &y) const
    {int c = coord (x, y); return c == -1 ? NULL : &data[c];}
  inline const struct map_cell * read_map (unsigned const int &ref) const
    {return &data[ref];}	// no error check

private:
  unsigned int _width, _height, _size;
  struct map_cell *data;
  const Communicate *comm;
};

#include "communicate.H"
#include "control.H"

#endif
